@@ -85,7 +85,7 @@ class Gui(abstract.LessonbasedGui):
         spacebox = gtk.HBox()
         hbox.pack_start(spacebox)
 
-        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer()
+        self.g_music_displayer = mpd.MusicDisplayer()
         self.g_music_displayer.set_size_request(100, -1)
         hbox.pack_start(self.g_music_displayer, False)
         spacebox = gtk.HBox()
@@ -162,7 +162,7 @@ class Gui(abstract.LessonbasedGui):
         trows = max([len(x) for x in self.m_t.m_P.m_props.values()]) + 2
         # The column headings
         for idx, label in enumerate(self.m_t.m_P.header.qprop_labels):
-            self.g_atable.attach(gtk.Label(label), idx * 2, idx * 2 + 1, 0, 1, 
+            self.g_atable.attach(gtk.Label(label), idx * 2, idx * 2 + 1, 0, 1,
                                  xoptions=gtk.FILL, yoptions=gtk.SHRINK,
                                  xpadding=gu.PAD_SMALL)
         # Then we create the buttons used to answer.
@@ -182,7 +182,7 @@ class Gui(abstract.LessonbasedGui):
             xpadding=0, ypadding=gu.PAD_SMALL)
         # The vertical separator between columns
         for idx in range(len(self.m_t.m_P.header.qprops)-1):
-            self.g_atable.attach(gtk.VSeparator(), 
+            self.g_atable.attach(gtk.VSeparator(),
             idx * 2 + 1, idx * 2 + 2, 0, trows,
             xoptions=gtk.FILL, yoptions=gtk.FILL,
             xpadding=0, ypadding=gu.PAD_SMALL)
