@@ -136,7 +136,7 @@ class Gui(abstract.Gui):
         ##############
         self.practise_box.set_spacing(gu.PAD)
         hbox = gu.bHBox(self.practise_box)
-        self.g_music_displayer = mpd.musicdisplayer.MusicDisplayer()
+        self.g_music_displayer = mpd.MusicDisplayer()
         self.g_music_displayer.clear()
         hbox.pack_start(self.g_music_displayer)
 
