@@ -32,6 +32,7 @@ import pango
 from solfege import cfg
 from solfege import osutils
 from solfege import soundcard
+from solfege import utils
 
 import solfege
 
@@ -227,17 +228,22 @@ class PercussionNameButton(gtk.Button, cfg.ConfigUtils):
     def entry_changed(self, widget, value):
         self.set_int(self.m_name, value + soundcard.first_percussion_int_value)
         self.set_label(soundcard.percussion_names[value])
-
+        t = utils.new_percussion_track()
+        t.note(8, value + soundcard.first_percussion_int_value)
+        t.note(16, value + soundcard.first_percussion_int_value)
+        t.note(16, value + soundcard.first_percussion_int_value)
+        t.note(4, value + soundcard.first_percussion_int_value)
+        soundcard.synth.play_track(t)
 
 class FlashBar(gtk.Frame):
     def __init__(self):
         gtk.Frame.__init__(self)
         self.set_shadow_type(gtk.SHADOW_IN)
-        #FIXME different gtk themes can make these values wrong
-        self.set_size_request(-1, 40)
         self.__stack = []
         self.__label = HarmonicProgressionLabel('')
-        self.add(self.__label)
+        self.__content = gtk.HBox()
+        self.__content.show()
+        self.add(self.__content)
         self.__timeout = None
         # The allocated size
         self.m_sx, self.m_sy = 0, 0
@@ -248,41 +254,57 @@ class FlashBar(gtk.Frame):
         show all the strings in stringlist.
         """
         for s in stringlist:
-            self.__label.set_text(s)
-            x, y =  self.__label.size_request()
-            self.m_sx = max(x, self.m_sx)
-            self.m_sy = max(y, self.m_sy)
-        self.__label.set_text("")
-        self.__label.set_size_request(self.m_sx, self.m_sy)
+            self.display(s)
+        self.empty()
     def delayed_flash(self, milliseconds, msg):
         gobject.timeout_add(milliseconds, lambda: self.flash(msg))
-    def flash(self, txt):
+    def empty(self):
+        for child in self.__content.get_children():
+            child.destroy()
+    def flash(self, txt, **kwargs):
         """Display a message that is automatically removed after some time.
         If we flash a new message before the old flashed message are removed,
         we old flashed message are removed.
         """
         if self.__timeout:
             gobject.source_remove(self.__timeout)
-        self.__label.set_size_request(-1, -1)
-        self.__label.set_text(txt)
-        sx, sy = self.__label.size_request()
-        self.m_sx = max(sx, self.m_sx)
-        self.m_sy = max(sy, self.m_sy)
-        self.__label.set_size_request(self.m_sx, self.m_sy)
+        self.display(txt, **kwargs)
         def f(self=self):
             self.__timeout = None
             if self.__stack:
-                self.__label.set_text(self.__stack[-1])
+                self.display(self.__stack[-1][0], **self.__stack[-1][1])
             else:
-                self.__label.set_text('')
+                self.empty()
         self.__timeout = gobject.timeout_add(2000, f)
-    def push(self, txt):
+    def push(self, txt, **kwargs):
         # stop any flashing before we push
+        self.display(txt, **kwargs)
+        self.__stack.append((txt, kwargs))
+    def display(self, txt, **kwargs):
+        self.empty()
+        r = re.compile("(\{\w+\})") # Unicode??
+        self.set_size_request(-1, -1)
+        for child in r.split(txt):
+            m = r.match(child)
+            if m:
+                varname = child[1:][:-1]
+                from solfege import lessonfilegui
+                w = lessonfilegui.new_labelobject(kwargs[varname])
+                w.show()
+                self.__content.pack_start(w, False)
+            else:
+                if child:
+                    label = gtk.Label(child)
+                    label.set_name("FlashBarLabel")
+                    self.__content.pack_start(label, False)
+                    label.show()
+        sx, sy = self.size_request()
+        self.m_sx = max(sx, self.m_sx)
+        self.m_sy = max(sy, self.m_sy)
+        self.set_size_request(self.m_sx, self.m_sy)
         if self.__timeout:
             gobject.source_remove(self.__timeout)
             self.__timeout = None
-        self.__stack.append(txt)
-        self.__label.set_text(txt)
     def pop(self):
         """If a message is being flashed right now, that flashing is
         not affected, but the message below the flashed message is removed.
@@ -291,19 +313,20 @@ class FlashBar(gtk.Frame):
             self.__stack.pop()
         if not self.__timeout:
             if self.__stack:
-                self.__label.set_text(self.__stack[-1])
+                self.display(self.__stack[-1][0],
+                           **self.__stack[-1][1])
             else:
-                self.__label.set_text('')
+                self.empty()
     def clear(self):
         self.__stack = []
         if not self.__timeout:
-            self.__label.set_text('')
-    def set(self, txt):
+            self.empty()
+    def set(self, txt, **kwargs):
         """
         Empty the stack of messages and display the string in txt.
         """
-        self.__stack = [txt]
-        self.__label.set_text(txt)
+        self.__stack = [(txt, kwargs)]
+        self.display(txt, **kwargs)
 
 class hig(object):
     SPACE_SMALL= 6
