@@ -1,5 +1,4 @@
 # GNU Solfege - free ear training software
-
 # vim: set fileencoding=utf-8 :
 # Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Tom Cato Amundsen
 #
@@ -28,7 +27,6 @@ import stat
 import subprocess
 import sys
 import textwrap
-import time
 
 import gobject
 
@@ -43,9 +41,8 @@ from solfege import uuid
 from solfege import xrandom
 from solfege.dataparser import istr
 from solfege.mpd import elems
-from solfege.mpd import mpdutils
-from solfege.mpd.duration import Duration
-from solfege.mpd.rat import Rat
+from solfege.mpd import Duration
+from solfege.mpd import Rat
 
 _test_mode = False
 
@@ -201,6 +198,10 @@ keywords = (
     'd8', 'p8', 'a8',
     'd9', 'm9', 'M9', 'a9',
     'd10', 'm10', 'M10', 'a10',
+    # clefs:
+    'treble', 'violin', 'french',
+    'subbass', 'bass', 'varbaritone',
+    'baritone', 'tenor', 'alto', 'mezzosoprano', 'soprano',
 )
 predef = {
     'tempo': (60, 4),
@@ -285,11 +286,11 @@ def rnc_markup_tokenizer(s):
     """
     [rn][mod1][num][\s-]
     """
-    rn_re = re.compile("""(?P<p1>b?[ivIV]+)
+    rn_re = re.compile(u"""(?P<p1>[b♭♯#]?[ivIV]+)
                           (?P<p2>[^\d\s-]*)
                           (?P<p3>[^\s-]*)
                           (?P<sep>(\s*-\s*|\s*))""",
-                       re.VERBOSE)
+                       re.VERBOSE|re.UNICODE)
     i = 0
     retval = []
     while i < len(s):
@@ -711,7 +712,7 @@ class Rvoice(VoiceCommon):
         except mpd.InvalidNotenameException, e:
             errpitch = self.m_musicdata.split("\n")[e.m_lineno][e.m_linepos1:e.m_linepos2]
             if lessonfile_ref.header.random_transpose[0]:
-                return "\\staff\\transpose %s\\relative %s{ %s }" % (
+                return "\\staff\\transpose %s\\relative %s{\n%s\n}" % (
                     lessonfile_ref.m_transpose.get_octave_notename(),
                     errpitch,#self.get_first_pitch().get_octave_notename(),
                     lex.m_string
@@ -723,7 +724,7 @@ class Rvoice(VoiceCommon):
         pitch.m_octave_i = 0
         lex.set_first_pitch(pitch)
         if lessonfile_ref.header.random_transpose[0]:
-            return "\\staff\\transpose %s\\relative %s{ %s }" % (
+            return "\\staff\\transpose %s\\relative %s{\n%s\n}" % (
                 lessonfile_ref.m_transpose.get_octave_notename(),
                 self.get_first_pitch().get_octave_notename(),
                 lex.m_string
@@ -747,6 +748,29 @@ class Rvoice(VoiceCommon):
             self.get_first_pitch().get_octave_notename(),
             self.m_musicdata,
             )
+    def get_err_context(self, exception, lessonfile_ref):
+        """
+        This functions expect exception.m_lineno, .m_linepos1 and .m_linepos2
+        to be set relative to the m_musicdata of the music object,
+        and not relative to the complete mpd string with added \staff{ } etc.
+
+        Return a twoline string showing what caused the exception.
+
+        Rvoice has its own implementation of get_err_context because we
+        will modify the first tone in the music, removing the octave
+        change characters (' and ,) and thus changing the lenght
+        of the string. So we need to adjust the placement of the
+        error marker (^^^^^).
+        """
+        try:
+            delta = abs(self.get_first_pitch().m_octave_i)
+        except mpd.InvalidNotenameException, e:
+            delta = 0
+        if getattr(exception, 'm_obj_lineno', False) is not False:
+            return "\n".join((self.m_musicdata.split("\n")[exception.m_obj_lineno],
+            " " * (exception.m_linepos1 + delta) + "^" * (exception.m_linepos2 - exception.m_linepos1)))
+        return self._mpd_err_context(exception, lessonfile_ref)
+
 
 class Satb(ChordCommon):
     def __init__(self, musicdata):
@@ -979,19 +1003,21 @@ class Music(MpdTransposable):
     def get_err_context(self, exception, lessonfile_ref):
         """
         Return a twoline string showing what caused the exception.
+        This method will report wrong error location if there are
+        more than one \staff command in a line.
         """
         first = exception.m_linepos1
         last = exception.m_linepos2
+        tr = r'\transpose %s' % lessonfile_ref.m_transpose.get_octave_notename()
+        delta = len(tr)
+        lines = self.m_musicdata.split("\n")
         if lessonfile_ref.header.random_transpose[0]:
-            s = self.m_musicdata.replace(r'\staff',
-               r'\staff\transpose %s' % lessonfile_ref.m_transpose.get_octave_notename())
-            s = s.replace(r'\addvoice',
-               r'\addvoice\transpose %s' % lessonfile_ref.m_transpose.get_octave_notename())
-            i = len(s) - len(self.m_musicdata)
-        else:
-            i = 0
-        first = exception.m_linepos1 - i
-        last = exception.m_linepos2 - i
+            s = self.m_musicdata.replace(r'\staff', r'\staff%s' % tr)
+            s = s.replace(r'\addvoice', r'\addvoice%s' % tr)
+            if (r'\staff' in lines[exception.m_lineno]
+                or r'\addvoice' in lines[exception.m_lineno]):
+                first = exception.m_linepos1 - delta
+                last = exception.m_linepos2 - delta
         return "\n".join((self.m_musicdata.split("\n")[exception.m_lineno],
             " " * first + "^" * (last - first)))
     def complete_to_musicdata_coords(self, lessonfile_ref, exception):
@@ -1801,7 +1827,7 @@ class InfoCache(object):
                 'replaces': p.header.get('replaces', []),
         }
         except KeyError:
-            logging.debug("InfoCache.parse_file: FileNotLessonfile(%s)" % filename)
+            logging.debug("InfoCache.parse_file: FileNotLessonfile(%s)", filename)
             raise self.FileNotLessonfile(filename)
     def iter_parse_all_files(self):
         """
@@ -1837,7 +1863,7 @@ class InfoCache(object):
         Parse all standard lesson files and the user_lessonfiles.
         Will not check if reparse is necessary.
         """
-        logging.debug("parse_all_files(when_idle=%s)" % when_idle)
+        logging.debug("parse_all_files(when_idle=%s)", when_idle)
         if when_idle:
             self._lessonfiles_iterator = self.iter_parse_all_files()
             def on_idle_parse():
@@ -1864,7 +1890,7 @@ class InfoCache(object):
             return
         mtime = os.path.getmtime(dir)
         assert isinstance(dir, unicode)
-        logging.debug("cond_parse_dir(%s) mtime=%s" % (dir, mtime))
+        logging.debug("cond_parse_dir(%s) mtime=%s", dir, mtime)
         if dir not in self._dir_mtime:
             for filename in os.listdir(dir):
                 fn = os.path.join(dir, filename)
@@ -1874,10 +1900,10 @@ class InfoCache(object):
                 file_mtime = os.path.getmtime(uri_expand(fn))
                 if fn not in self._data or file_mtime > self._data[fn]['mtime']:
                     try:
-                        logging.debug(" will parse %s. in data:%s" % (fn, fn in self._data))
+                        logging.debug(" will parse %s. in data:%s", fn, fn in self._data)
                         self.parse_file(fn, file_mtime)
                     except self.InfoCacheException:
-                        logging.debug(" exception, not parsed: %s" % fn)
+                        logging.debug(" exception, not parsed: %s", fn)
                         pass
     def iter_user_files(self):
         """
