@@ -24,14 +24,15 @@ from __future__ import absolute_import
 # noteheads. So I think it should be given the created NoteHeadEngravers
 # and then shift the notes.
 
+import copy
 import re
 import weakref
 
 from solfege.mpd import _exceptions
 from solfege.mpd import const
-from solfege.mpd import Rat
-from solfege.mpd import Duration
-from solfege.mpd import MusicalPitch
+from solfege.mpd.rat import Rat
+from solfege.mpd.duration import Duration
+from solfege.mpd.musicalpitch import MusicalPitch
 
 #FIXME duplicate of code in Lexer
 re_melodic = re.compile(r"""(?x)
@@ -94,23 +95,22 @@ class Clef(object):
     def steps_to_ylinepos(self, steps):
         return 7-self.clefdata[self.m_name][self.POS] - steps + self.m_octaviation
     def an_to_ylinepos(self, an):
-        def notename_to_ylinepos(n, clef):
+        def notename_to_ylinepos(n):
             n = MusicalPitch.new_from_notename(n)
-            i = n.steps()
-            return self.steps_to_ylinepos(i)
+            return self.steps_to_ylinepos(n.steps())
         if an[-2:] == 'es':
             l = 3
             h = -3
         else:
             l = 1
             h = -5
-        i = notename_to_ylinepos(an, self)
+        i = notename_to_ylinepos(an)
         while i > l:
             an = an + "'"
-            i =  notename_to_ylinepos(an, self)
+            i =  notename_to_ylinepos(an)
         while i < h:
             an = an + ","
-            i = notename_to_ylinepos(an, self)
+            i = notename_to_ylinepos(an)
         return i
 
 class TimeSignature(object):
@@ -222,7 +222,7 @@ class Stem(list):
         assert isinstance(elemlist, list)
         assert stemdir in (const.UP, const.DOWN, const.BOTH)
         if [x for x in elemlist if x.m_duration != elemlist[0].m_duration]:
-            raise NotEqualLengthException()
+            raise self.NotEqualLengthException()
         list.__init__(self, elemlist)
         self.w_parent = weakref.ref(parent)
         for note in self:
@@ -239,23 +239,35 @@ class Voice(HasParent):
         pass
     class NoChordsInRhythmStaffException(Exception):
         pass
-    def __init__(self, parent):
-        HasParent.__init__(self, parent)
-        # The timelen of the Voice
-        self.m_length = Rat(0, 1)
-        self.m_tdict = {}
+    class NotUnisonException(Exception):
+        pass
     class BarFullException(Exception):
         """
         Exception raised if we try to add a note or rest that is longer than
         the available time in the bar. FIXME: maybe name better, like
         NotEnoughtTimeException or similar.
         """
-        pass
+        def __unicode__(self):
+            return u"There is not enough space left in the bar"
     class NoteDontBelongHere(Exception):
         """
         raised if we try to beam notes that does not belong to this voice.
         """
         pass
+    def __init__(self, parent):
+        HasParent.__init__(self, parent)
+        # The timelen of the Voice
+        self.m_length = Rat(0, 1)
+        self.m_tdict = {}
+    def copy(self, parent):
+        """
+        Return a copy of this Voice object. We make a copy of the dict and
+        the m_length variable, but the dict revers to the same object.
+        """
+        ret = Voice(parent)
+        ret.m_length = Rat(self.m_length.m_num, self.m_length.m_den)
+        ret.m_tdict = self.m_tdict.copy()
+        return ret
     def append(self, elem, stemdir=const.BOTH):
         """
         elem - either a MusicElement or a list of MusicElements
@@ -363,13 +375,42 @@ class Voice(HasParent):
             max_free = bp.get_free_time()
             #flytt til bar-class
             delta = elem.m_duration.get_rat_value() - self.m_tdict[timepos]['elem'][0].m_duration.get_rat_value()
+            def fix_ties(prev_timepos, cur_timepos, next_timepos):
+                prev_e = self.m_tdict[prev_timepos]['elem'][0] if prev_timepos else None
+                cur_e = self.m_tdict[cur_timepos]['elem'][0]
+                next_e = self.m_tdict[next_timepos]['elem'][0] if next_timepos else None
+                if isinstance(elem, Note):
+                    t = None
+                    if prev_timepos and isinstance(prev_e, Note) and prev_e.m_tieinfo in ('go', 'start'):
+                        t = 'end'
+                    if isinstance(next_e, Note) and next_e.m_tieinfo in ('go', 'end'):
+                        if t == 'end':
+                            t = 'go'
+                        else:
+                            t = 'start'
+                    cur_e.m_tieinfo = t
+                else:
+                    assert isinstance(elem, Rest)
+                    if prev_timepos and isinstance(prev_e, Note):
+                        if prev_e.m_tieinfo == 'go':
+                            prev_e.m_tieinfo = 'end'
+                        elif prev_e.m_tieinfo == 'start':
+                            prev_e.m_tieinfo = None
+                    if isinstance(next_e, Note):
+                        if next_e.m_tieinfo == 'go':
+                            next_e.m_tieinfo = 'start'
+                        elif next_e.m_tieinfo == 'end':
+                            next_e.m_tieinfo = None
+
             if insert_mode:
                 if max_free >= elem.m_duration.get_rat_value():
-                    self.m_tdict[timepos + Rat(1, 1000000)] = self.m_tdict[timepos]
+                    tmp_timepos = timepos + Rat(1, 1000000)
+                    self.m_tdict[tmp_timepos] = self.m_tdict[timepos]
                     del self.m_tdict[timepos]
                     bp.remove_trailing(elem.m_duration.get_rat_value())
                     stem = Stem(self, [elem], const.UP)
                     self.set_elem(stem, timepos)
+                    fix_ties(self.get_prev_timepos(timepos), timepos, tmp_timepos)
                     bp.remove_skips()
                     bp.repack()
                     bp.fill_skips()
@@ -383,6 +424,7 @@ class Voice(HasParent):
                         bp.remove_trailing(delta)
                     stem = Stem(self, [elem], const.UP)
                     self.set_elem(stem, timepos)
+                    fix_ties(self.get_prev_timepos(timepos), timepos, self.get_next_timepos(timepos))
                     bp.remove_skips()
                     bp.repack()
                     bp.fill_skips()
@@ -419,6 +461,23 @@ class Voice(HasParent):
             return v[start_i:end_i]
         else:
             return v[start_i:]
+    def get_time_pitch_list(self, bpm):
+        """
+        Return a list of tuples (pitch, duration-in-seconds) of the tones
+        and rests in the voice. -1 is used for pitch for rests.
+        """
+        ret = []
+        for timepos in sorted(self.m_tdict.keys()):
+            # stem is a Stem or [Rest]
+            stem = self.m_tdict[timepos]['elem']
+            if len(stem) != 1:
+                raise Voice.NotUnisonException()
+            dur = float(stem[0].m_duration.get_rat_value()) * bpm[1] / bpm[0] * 60
+            if isinstance(stem[0], Note):
+                ret.append((stem[0].m_musicalpitch.semitone_pitch(), dur))
+            elif isinstance(stem[0], Rest):
+                ret.append((-1, dur))
+        return ret
     def beam(self, notes):
         """
         Notes is a list of Note objects, once Note object from each stem
@@ -602,7 +661,6 @@ class Voice(HasParent):
         if i +1 < len(v):
             return v[i + 1]
     def get_timelist(self):
-        curpos = Rat(0, 1)
         retval = []
         for timepos in sorted(self.m_tdict.keys()):
             if isinstance(self.m_tdict[timepos]['elem'][0], Rest):
@@ -795,7 +853,15 @@ class _StaffCommon(HasParent):
         HasParent.__init__(self, parent)
         self.m_voices = []
         self.add_voice()
+        # I think the only things stored in Staff.m_tdict are "clef" and
+        # "keysig". We don't store time signature changes where, since
+        # Score.m_bars take care about that.
         self.m_tdict = {}
+    def copy(self, parent):
+        staff = self.__class__(parent)
+        staff.m_voices = [v.copy(staff) for v in self.m_voices]
+        staff.m_tdict = self.m_tdict.copy()
+        return staff
     def add_voice(self):
         self.m_voices.append(Voice(self))
         self.w_score().create_shortcuts()
@@ -825,11 +891,10 @@ class _StaffCommon(HasParent):
                         data[endpos] = {'start': set(), 'end': set()}
                     data[endpos]['end'].add(voice_idx)
         retval = []
-        start_time = None
         v = sorted(data)[:]
         ppos = Rat(0, 1)
         voices = set()
-        for idx, timepos in enumerate(v):
+        for timepos in v:
             if data[timepos]['start'] and data[timepos]['end']:
                 for voice in data[timepos]['end']:
                     voices.remove(voice)
@@ -882,7 +947,7 @@ class RhythmStaff(_StaffCommon):
         _StaffCommon.__init__(self, parent)
     def add_voice(self):
         if len(self.m_voices) == 1:
-            raise OnlyOneVoiceException()
+            raise self.OnlyOneVoiceException()
         _StaffCommon.add_voice(self)
     def set_key_signature(self, keysig, timepos):
         """
@@ -892,9 +957,22 @@ class RhythmStaff(_StaffCommon):
 
 
 class Score(object):
+    class ConcatException(Exception):
+        pass
+    class StaffCountException(ConcatException):
+        pass
+    class StaffTypeException(ConcatException):
+        pass
+    class VoiceCountException(ConcatException):
+        pass
     def __init__(self):
         self.m_staffs = []
         self.m_bars = []
+    def copy(self):
+        score = Score()
+        score.m_staffs = [s.copy(score) for s in self.m_staffs]
+        score.m_bars = copy.deepcopy(self.m_bars)
+        return score
     def add_staff(self, staff_class=Staff):
         self.m_staffs.append(staff_class(self))
         self.create_shortcuts()
@@ -946,7 +1024,7 @@ class Score(object):
         Return the bar timepos is within. Raise IndexError if timepos
         is after the last bar.
         """
-        for idx, bar in enumerate(self.m_bars):
+        for bar in self.m_bars:
             if bar.m_timepos <= timepos < bar.end():
                 return bar
             if bar.m_timepos > timepos:
@@ -969,11 +1047,10 @@ class Score(object):
                                 data[endpos] = {'start': set(), 'end': set()}
                             data[endpos]['end'].add((staff_idx, voice_idx))
         retval = []
-        start_time = None
         v = sorted(data)[:]
         ppos = Rat(0, 1)
         voices = set()
-        for idx, timepos in enumerate(v):
+        for timepos in v:
             if data[timepos]['start'] and data[timepos]['end']:
                 for voice in data[timepos]['end']:
                     voices.remove(voice)
@@ -999,4 +1076,72 @@ class Score(object):
                     retval.append([True, timepos - ppos])
                     ppos = timepos
         return retval
-
+    @staticmethod
+    def concat(s1, s2):
+        """
+        Concatenate the two scores, and return a new score. Both scores
+        need to have the exact same staff and voice layout.
+        """
+        assert isinstance(s1, Score)
+        assert isinstance(s2, Score)
+        if len(s1.m_staffs) != len(s2.m_staffs):
+            raise Score.StaffCountException()
+        if [type(x) for x in s1.m_staffs] != [type(x) for x in s2.m_staffs]:
+            raise Score.StaffTypeException()
+        for st1, st2 in zip(s1.m_staffs, s2.m_staffs):
+            if len(st1.m_voices) != len(st2.m_voices):
+                raise Score.VoiceCountException()
+        ret = s1.copy()
+        if not s1.m_staffs:
+            return s1
+        # do the adding
+        for bar in s2.m_bars:
+            bar.m_timepos = ret.m_bars[-1].end()
+            ret.m_bars.append(bar)
+        ret.create_shortcuts()
+        if s1.m_bars:
+            start = s1.m_bars[-1].end()
+        else:
+            start = Rat(0, 1)
+        s2.create_shortcuts() # FIXME why?
+        for staff_idx, staff in enumerate(s1.m_staffs):
+            for voice_idx in range(len(staff.m_voices)):
+                for k in s2.m_staffs[staff_idx].m_voices[voice_idx].m_tdict:
+                    ret.m_staffs[staff_idx].m_voices[voice_idx].m_tdict[k + start] = s2.m_staffs[staff_idx].m_voices[voice_idx].m_tdict[k]
+        return ret
+    @staticmethod
+    def concat2(s1, s2):
+        """
+        Return a new Score object concatenating the two scores. This is
+        intended return value is intended for playback only, since the
+        staffs placed below each other. So the first score will have empty
+        bars at the end, and the last score will have empty bars at the
+        beginning.
+        """
+        assert isinstance(s1, Score)
+        assert isinstance(s2, Score)
+        ret = s1.copy()
+        if s1.m_bars:
+            start = s1.m_bars[-1].end()
+        else:
+            start = Rat(0, 1)
+        for bar in s2.m_bars:
+            ret.m_bars.append(Bar(bar.m_timesig, ret.m_bars[-1].end()))
+        for staff_idx, staff in enumerate(s2.m_staffs):
+            ret.add_staff(staff_class=staff.__class__)
+            for k in staff.m_tdict:
+                ret.m_staffs[-1].m_tdict[start + k] = staff.m_tdict[k]
+            for voice_idx in range(len(staff.m_voices)):
+                if voice_idx != 0:
+                    ret.m_staffs[-1].add_voice()
+                # This line make the music from sc2 continue after the
+                # point where the music from sc1 ends.
+                ret.m_staffs[-1].m_voices[-1].m_length = s1.m_bars[-1].end()
+                for elem in s2.m_staffs[staff_idx].m_voices[voice_idx]:
+                    ret.m_staffs[-1].m_voices[-1].append(elem['elem'])
+        ret.create_shortcuts()
+        return ret
+    def __deepcopy__(self, memo):
+        ret = Score()
+        ret.m_bars = copy.deepcopy(self.m_bars)
+        return ret
