@@ -21,8 +21,6 @@ import sys
 
 import gtk
 
-from solfege import cfg
-
 class BaseIconFactory(gtk.IconFactory):
     def __init__(self, widget, datadir):
         gtk.IconFactory.__init__(self)
@@ -43,7 +41,7 @@ class EditorIconFactory(BaseIconFactory):
     """
     def __init__(self, widget, datadir):
         BaseIconFactory.__init__(self, widget, datadir)
-        icons = {'solfege-icon': "graphics/solfege.png", #FIXME new icon
+        icons = {'solfege-icon': "graphics/solfege.svg",
             'solfege-sharp': "graphics/sharp.png",
             'solfege-double-sharp': "graphics/double-sharp.png",
             'solfege-flat': "graphics/flat.png",
@@ -57,7 +55,7 @@ class EditorIconFactory(BaseIconFactory):
 class SolfegeIconFactory(BaseIconFactory):
     def __init__(self, widget, datadir):
         BaseIconFactory.__init__(self, widget, datadir)
-        icon_list = ['happyface', 'sadface', 
+        icon_list = ['happyface', 'sadface',
             'rhythm-c12c12c12', 'rhythm-c12c12r12', 'rhythm-c12r12c12',
             'rhythm-c16c16c16c16', 'rhythm-c16c16c8', 'rhythm-c16c8c16',
             'rhythm-c16c8.', 'rhythm-c4', 'rhythm-c8c16c16', 'rhythm-c8.c16',
@@ -67,7 +65,7 @@ class SolfegeIconFactory(BaseIconFactory):
             'rhythm-r8c16c16', 'rhythm-r8c8', 'rhythm-r8r16c16',
             'rhythm-wrong']
         d = {}
-        d['solfege-icon'] = 'graphics/solfege.png'
+        d['solfege-icon'] = 'graphics/solfege.svg'
         for iname in icon_list:
             d['solfege-%s' % iname] = os.path.join(u"graphics", iname) + ".png"
         self.add_icons(d)
